#!/bin/zsh

# Process: Do the whole process from minimap_W0_Matthews

# cd ~/[base directory in which to run scripts and place files]
# permissions with chmod u+x _Paper_analysis/vs_self_output/Paper_AllvsSelfv2_part1.sh
# run with ./_Paper_analysis/vs_self_output/Paper_AllvsSelfv2_part1.sh

# Here all relevant input files should be in the folder in the cd line above:
#     EBV_annotations; Guppy_6_output (fastq) ; EBV_FAST5; Aaron_script

#can be adapted by replacing AllvsSelf (or tag marked beow)
# rename destination files in communal folders to avoid overwriting read files: EBV_Fast5 -> EBV_Fast5;  EBV_readIDs -> EBV_readIDs
# Folders required from run directory: ./Guppy_6_output containing fastq; EBV_Fast5 containing fast5; SAM_files; SAM_unq; Matthews _script containing two Matthews perl scripts; Aaron_script containing three W0 R scripts; W0_SAM containing folders W0_first, 

#Concatenate original fastq reads into single file for mapping.

# to prepare a new run: copy _Paper_analysis/vs_self_output_empty to new folder, and mkdir _Paper_analysis/vs_self_output/SAM_files 

#cat  ./Guppy_6_output/AllFASTq/* > ./Guppy_6_output/Guppy6_Allpassreads.fastq.gz

# first map reads. Fasta: MR11_oriPMlu.fa reads: ./Guppy_6_output/AllFASTq ; tag: AllvsSelf
#alternative is align to self separately, and then concatenate the SAM files. 
# minimap2 -ax splice -uf -k14  --junc-bed EBV_annotations/EBV_annotations_paper/pHB9_2023v1.bed EBV_annotations/EBV_annotations_paper/MR11_oriPMlu.fa ./Guppy_6_output/Guppy6_Allpassreads.fastq.gz > _Paper_analysis/vs_self_output/SAM_files/AllvsSelf.sam

# switch to oriP_Mlu template, need to change: _Paper_analysis/vs_self_output change to _Paper_analysis/vs_self_output
# change annotation files to new numbers: EBNAexons_pHB9_oriPMlu.bed -> EBNAexons_pHB9_oriPMlu.bed ; Features_table_pHB9_oriPMlu.txt -> Features_table_pHB9_oriPMlu.txt MR11_oriPMlu.fa -> MR11_oriPMlu.fa; pHB9_oriPMlu.fa -> pHB9_oriPMlu.fa WTw_2023.fa -> WTw_oriPMlu.fa ; MR11_noATG.fa ->  MR11_noATG ;  pHB9_oriPMluv4.bed -> pHB9_oriPMluv4.bed (made with agat_convert_sp_gff2bed.pl --gff EBV_annotations/pHB9_oriPMluv3.gff3 -o EBV_annotations/pHB9_oriPMluv4.bed)


mkdir ./_Paper_analysis/vs_self_output/SAM_files
mkdir ./_Paper_analysis/vs_self_output/W0_SAM
mkdir ./_Paper_analysis/vs_self_output/SAM_unq
mkdir ./_Paper_analysis/vs_self_output/W0_SAM/W0_all_reads
mkdir ./_Paper_analysis/vs_self_output/W0_SAM/W0_ends_not_assigned
mkdir ./_Paper_analysis/vs_self_output/W0_SAM/W0_first
mkdir ./_Paper_analysis/vs_self_output/W0_SAM/W0_full_length
# mkdir ./_Paper_analysis/vs_self_output/W0_SAM/W0_transcript_variation
mkdir ./_Paper_analysis/vs_self_output/W0_SAM/W0_5p_intact
mkdir ./_Paper_analysis/vs_self_output/W0_SAM/W0_3p_intact
mkdir ./_Paper_analysis/vs_self_output/Matthews_script
mkdir ./_Paper_analysis/vs_self_output/Matthews_script/Script_out
mkdir ./_Paper_analysis/vs_self_output/Matthews_script/Nanopolish 

# align reads to EBV genome. Guided by annotaiton and splice junction consensus rules.

minimap2 -ax splice -uf -k14 --junc-bed EBV_annotations/EBV_annotations_paper/pHB9_oriPMluv4.bed EBV_annotations/EBV_annotations_paper/pHB9_oriPMlu.fa ./Guppy_6_output/HB9/HB9_Guppy6_Allpassreads.fastq.gz > _Paper_analysis/vs_self_output/SAM_files/HB9vsSelf.sam

minimap2 -ax splice -uf -k14 --junc-bed EBV_annotations/EBV_annotations_paper/pHB9_oriPMluv4.bed EBV_annotations/EBV_annotations_paper/WTw_oriPMlu.fa ./Guppy_6_output/WTw-A11/WTw_Guppy6_Allpassreads.fastq.gz > _Paper_analysis/vs_self_output/SAM_files/WTwvsSelf.sam

minimap2 -ax splice -uf -k14 --junc-bed EBV_annotations/EBV_annotations_paper/pHB9_oriPMluv4.bed EBV_annotations/EBV_annotations_paper/MR11_oriPMlu.fa ./Guppy_6_output/M11-AJ2/M11-AJ2_Guppy6_Allpassreads.fastq.gz > _Paper_analysis/vs_self_output/SAM_files/M11-AJ2vsSelf.sam

minimap2 -ax splice -uf -k14 --junc-bed EBV_annotations/EBV_annotations_paper/pHB9_oriPMluv4.bed EBV_annotations/EBV_annotations_paper/MR11_oriPMlu.fa ./Guppy_6_output/M11-MJ2/M11-MJ2_Guppy6_Allpassreads.fastq.gz > _Paper_analysis/vs_self_output/SAM_files/M11-MJ2vsSelf.sam

# convert SAMs to sorted BAMs -  the xxx_sort.bam file is the main input file for later steps
samtools view -b _Paper_analysis/vs_self_output/SAM_files/HB9vsSelf.sam > _Paper_analysis/vs_self_output/SAM_files/HB9vsSelf.bam 
samtools view -F 0x04 -b _Paper_analysis/vs_self_output/SAM_files/HB9vsSelf.bam > _Paper_analysis/vs_self_output/SAM_files/HB9vsSelf_mapped.bam
samtools sort _Paper_analysis/vs_self_output/SAM_files/HB9vsSelf_mapped.bam > _Paper_analysis/vs_self_output/SAM_files/HB9vsSelf_sort.bam 
samtools index -b _Paper_analysis/vs_self_output/SAM_files/HB9vsSelf_sort.bam
echo " Mapped reads in HB9="
# remove secondary alignments and other QC-failures and count read numbers:
samtools view -c -F 260 _Paper_analysis/vs_self_output/SAM_files/HB9vsSelf_sort.bam 
# make list of fastq IDs for this sample, to later use for de-convoluting pooled reads.
samtools view _Paper_analysis/vs_self_output/SAM_files/HB9vsSelf.sam | cut -f 1 > ./Guppy_6_output/HB9vsSelfEBV_readIDs.txt


samtools view -b _Paper_analysis/vs_self_output/SAM_files/WTwvsSelf.sam > _Paper_analysis/vs_self_output/SAM_files/WTwvsSelf.bam 
samtools view -F 0x04 -b _Paper_analysis/vs_self_output/SAM_files/WTwvsSelf.bam > _Paper_analysis/vs_self_output/SAM_files/WTwvsSelf_mapped.bam
samtools sort _Paper_analysis/vs_self_output/SAM_files/WTwvsSelf_mapped.bam > _Paper_analysis/vs_self_output/SAM_files/WTwvsSelf_sort.bam 
samtools index -b _Paper_analysis/vs_self_output/SAM_files/WTwvsSelf_sort.bam
echo " Mapped reads in WTw="
samtools view -c -F 260 _Paper_analysis/vs_self_output/SAM_files/WTwvsSelf_sort.bam
# make list of fastq IDs 
samtools view _Paper_analysis/vs_self_output/SAM_files/WTwvsSelf.sam | cut -f 1 > ./Guppy_6_output/WTwvsSelfEBV_readIDs.txt

samtools view -b _Paper_analysis/vs_self_output/SAM_files/M11-AJ2vsSelf.sam > _Paper_analysis/vs_self_output/SAM_files/M11-AJ2vsSelf.bam 
samtools view -F 0x04 -b _Paper_analysis/vs_self_output/SAM_files/M11-AJ2vsSelf.bam > _Paper_analysis/vs_self_output/SAM_files/M11-AJ2vsSelf_mapped.bam
samtools sort _Paper_analysis/vs_self_output/SAM_files/M11-AJ2vsSelf_mapped.bam > _Paper_analysis/vs_self_output/SAM_files/M11-AJ2vsSelf_sort.bam 
samtools index -b _Paper_analysis/vs_self_output/SAM_files/M11-AJ2vsSelf_sort.bam
echo " Mapped reads in M11-AJ2="
samtools view -c -F 260 _Paper_analysis/vs_self_output/SAM_files/M11-AJ2vsSelf_sort.bam
# make list of fastq IDs
samtools view _Paper_analysis/vs_self_output/SAM_files/M11-AJ2vsSelf.sam | cut -f 1 > ./Guppy_6_output/M11-AJ2vsSelfEBV_readIDs.txt

samtools view -b _Paper_analysis/vs_self_output/SAM_files/M11-MJ2vsSelf.sam > _Paper_analysis/vs_self_output/SAM_files/M11-MJ2vsSelf.bam 
samtools view -F 0x04 -b _Paper_analysis/vs_self_output/SAM_files/M11-MJ2vsSelf.bam > _Paper_analysis/vs_self_output/SAM_files/M11-MJ2vsSelf_mapped.bam
samtools sort _Paper_analysis/vs_self_output/SAM_files/M11-MJ2vsSelf_mapped.bam > _Paper_analysis/vs_self_output/SAM_files/M11-MJ2vsSelf_sort.bam 
samtools index -b _Paper_analysis/vs_self_output/SAM_files/M11-MJ2vsSelf_sort.bam
echo " Mapped reads in M11-MJ2="
samtools view -c -F 260 _Paper_analysis/vs_self_output/SAM_files/M11-MJ2vsSelf_sort.bam
# make list of fastq IDs for later splitting merged file
samtools view _Paper_analysis/vs_self_output/SAM_files/M11-MJ2vsSelf.sam | cut -f 1 > ./Guppy_6_output/M11-MJ2vsSelfEBV_readIDs.txt

# Merge all read IDs into total EBV read ID list.

cat ./Guppy_6_output/M11-MJ2vsSelfEBV_readIDs.txt ./Guppy_6_output/M11-AJ2vsSelfEBV_readIDs.txt ./Guppy_6_output/WTwvsSelfEBV_readIDs.txt ./Guppy_6_output/HB9vsSelfEBV_readIDs.txt > ./Guppy_6_output/AllvsSelfEBV_readIDs.txt


# removes all of the intermediate files, including the orignal SAM file. 
rm _Paper_analysis/vs_self_output/SAM_files/*_mapped.bam
rm _Paper_analysis/vs_self_output/SAM_files/M11-MJ2vsSelf.sam
rm _Paper_analysis/vs_self_output/SAM_files/M11-AJ2vsSelf.sam
rm _Paper_analysis/vs_self_output/SAM_files/M11-MJ2vsSelf.bam
rm _Paper_analysis/vs_self_output/SAM_files/M11-AJ2vsSelf.bam
rm _Paper_analysis/vs_self_output/SAM_files/HB9vsSelf.bam
rm _Paper_analysis/vs_self_output/SAM_files/HB9vsSelf.sam
rm _Paper_analysis/vs_self_output/SAM_files/WTwvsSelf.bam
rm _Paper_analysis/vs_self_output/SAM_files/WTwvsSelf.sam



# merge & index the four separate BAM files into a single BAM file add -f to force file overwrite. merge is expected to produce sorted bam.
samtools merge -f _Paper_analysis/vs_self_output/SAM_files/AllvsSelf_sort.bam _Paper_analysis/vs_self_output/SAM_files/M11-MJ2vsSelf_sort.bam _Paper_analysis/vs_self_output/SAM_files/M11-AJ2vsSelf_sort.bam _Paper_analysis/vs_self_output/SAM_files/WTwvsSelf_sort.bam _Paper_analysis/vs_self_output/SAM_files/HB9vsSelf_sort.bam
samtools index -b _Paper_analysis/vs_self_output/SAM_files/AllvsSelf_sort.bam

#report mapped reads
echo " Mapped reads in AllvsSelf="
#remove duplicates and AQC fails - should be redundant
samtools view -c -F 260 _Paper_analysis/vs_self_output/SAM_files/AllvsSelf_sort.bam

#Now make headered and headerless unique read files for Aaron script, and transfer to Matthews script

samtools view -F 2304 -b _Paper_analysis/vs_self_output/SAM_files/AllvsSelf_sort.bam > _Paper_analysis/vs_self_output/SAM_unq/AllvsSelf_unq.bam
samtools index -b _Paper_analysis/vs_self_output/SAM_unq/AllvsSelf_unq.bam

#report mapped reads
echo " Mapped Unique reads in AllvsSelf = "
samtools view -c -F 260 _Paper_analysis/vs_self_output/SAM_unq/AllvsSelf_unq.bam

#Convert to headered SAM for later
samtools view -h ./_Paper_analysis/vs_self_output/SAM_unq/AllvsSelf_unq.bam > ./_Paper_analysis/vs_self_output/SAM_unq/AllvsSelf_unq.sam

# trim columns from the end of the file and saves as header-less SAM - no longer required. 
# samtools view ./_Paper_analysis/vs_self_output/SAM_unq/AllvsSelf_unq.sam | cut -f 1-10 > ./_Paper_analysis/vs_self_output/SAM_unq/AllvsSelf_unq_trim.sam

#Cd /Volumes/PC_EXTERNAL/RNAseq_minION-fast5_Guppy-fastq/Bristol-RNAseq
#command calls the fast5 folder that is here. 
#mkdir /Users/rew/Library/CloudStorage/OneDrive-ImperialCollegeLondon/_Data/RNA-seq-robOnedrive/Nanopore_RNA-seq/EBV_Fast5
# optional: extract only the reads that align to EBV to make it quicker to run EBV-only analyses
#fast5_subset --input /Volumes/PC_EXTERNAL/RNAseq_minION-fast5_Guppy-fastq/Bristol-RNAseq/fast5 --save_path /Users/rew/Library/CloudStorage/OneDrive-ImperialCollegeLondon/_Data/RNA-seq-robOnedrive/Nanopore_RNA-seq/EBV_Fast5 --read_id_list /Users/rew/Library/CloudStorage/OneDrive-ImperialCollegeLondon/_Data/RNA-seq-robOnedrive/Nanopore_RNA-seq/Guppy_6_output/AllvsSelfEBV_readIDs.txt --batch_size 100 --recursive

